From 412cfdc292c8a4087b3656138953b17df70e382f Mon Sep 17 00:00:00 2001
From: "Robert A. Burks" <rburksdev@gmail.com>
Date: Thu, 7 Mar 2024 18:47:37 -0500
Subject: [PATCH 11/31] Changed 'trapped_write' flag mechanics and added
 extensive commentary

* src/eval.c (defvaralias):
'trapped_write' no longer needs to be maintained along variable alias
chains as previous changes have made checking occur at the chain's end.
---
 src/eval.c | 50 +++++++++++++++++++++++++++++++++++++++++++++-----
 1 file changed, 45 insertions(+), 5 deletions(-)

diff --git a/src/eval.c b/src/eval.c
index 5497cc6031e..f7135e8e898 100644
--- a/src/eval.c
+++ b/src/eval.c
@@ -576,8 +576,10 @@ DEFUN ("defvaralias", Fdefvaralias, Sdefvaralias, 2, 3, 0,
   CHECK_SYMBOL (new_alias);
   CHECK_SYMBOL (base_variable);
 
-  if (SYMBOL_CONSTANT_P (new_alias))
-    /* Making it an alias effectively changes its value.  */
+  if (SYMBOL_CONSTANT_P (Findirect_variable (new_alias)))
+  /* Making it an alias effectively changes its value.
+     This prevents re-aliasing any aliases of a constant and will
+     detect if any alias in the chain is re-aliased to a constant.  */
     error ("Cannot make a constant an alias: %s",
 	   SDATA (SYMBOL_NAME (new_alias)));
 
@@ -603,7 +605,7 @@ DEFUN ("defvaralias", Fdefvaralias, Sdefvaralias, 2, 3, 0,
       error ("Don't know how to make a buffer-local variable an alias: %s",
 	     SDATA (SYMBOL_NAME (new_alias)));
     case SYMBOL_PLAINVAL:
-    case SYMBOL_VARALIAS:
+    case SYMBOL_VARALIAS: /* Allow an alias to be re-aliased.  */
       break;
     default:
       emacs_abort ();
@@ -643,14 +645,52 @@ DEFUN ("defvaralias", Fdefvaralias, Sdefvaralias, 2, 3, 0,
 	       SDATA (SYMBOL_NAME (new_alias)));
   }
 
+  /* If an already watched variable is becoming an alias notify its
+     watchers, this doesn't apply to re-aliasing if the current base is
+     watched.  When a symbol becomes an alias it cannot be trapped
+     again itself.  Watcher adding/removing will occur at the end of
+     the alias chain.  Both symbols are declared_special and that can
+     only change via 'internal-make-var-not-special'. Once a variable
+     becomes an alias its redirect flag cannot be changed.  At this point
+     it can only be re-aliased.  We are cleaning up the plist in case a
+     user uses `symbol-plist' and is confused by watcher functions that
+     don't run and cannot be removed by the watcher functions.  Functions
+     such as `put', `get', and `symbol-plist' do not redirect.  */
   if (sym->u.s.trapped_write == SYMBOL_TRAPPED_WRITE)
-    notify_variable_watchers (new_alias, base_variable, Qdefvaralias, Qnil);
+    {
+      /* If a variable being made obsolete (i.e. new_alias) has a
+	 broken watcher it can fail here and the variable alias will
+	 not be created.  We should potentially condition-case and turn
+	 the error into a warning and proceed with alias creation.  */
+      notify_variable_watchers (new_alias, base_variable, Qdefvaralias, Qnil);
+
+      /* If new_alias has a watcher list clear it out.  */
+      Lisp_Object watchers = Fget (new_alias, Qwatchers);
+      if (!NILP (watchers))
+	sym->u.s.plist = plist_put (sym->u.s.plist, Qwatchers, Qnil);
+    }
+  /* SYMBOL_NOWRITE (i.e. constant) is checked for in other functions
+     at every step of variable alias redirection, it's only really
+     needed at the end.  Constants cannot be watched and an alias of a
+     constant cannot be re-aliased.  Therefore, it's a state that
+     cannot change once gained so we can assign it along the alias
+     chain. If an alias in the middle of a chain is re-aliased to a
+     constant its flag will be set and its aliases will be prevented
+     from being re-aliased, but their flags will not be explicitly set.
+     SYMBOL_(UN)TRAPPED_WRITE (i.e. watched) is to only be checked for
+     in other functions at the end of the variable alias redirection
+     chain.  Therefore, it will only be (un)set there. This makes
+     searching out obsolete variables to update them unneeded as watcher
+     notification depends only on the end of the chain.  */
+  if (SYMBOL_CONSTANT_P (Findirect_variable (base_variable)))
+    sym->u.s.trapped_write = SYMBOL_NOWRITE;
+  else
+    sym->u.s.trapped_write = SYMBOL_UNTRAPPED_WRITE;
 
   sym->u.s.declared_special = true;
   XSYMBOL (base_variable)->u.s.declared_special = true;
   sym->u.s.redirect = SYMBOL_VARALIAS;
   SET_SYMBOL_ALIAS (sym, XSYMBOL (base_variable));
-  sym->u.s.trapped_write = XSYMBOL (base_variable)->u.s.trapped_write;
   LOADHIST_ATTACH (new_alias);
   /* Even if docstring is nil: remove old docstring.  */
   Fput (new_alias, Qvariable_documentation, docstring);
-- 
2.34.1

